//+------------------------------------------------------------------+
//|                                               heikin-ashi-ea.mq5 |
//|                                                           andrea |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                                                  |
//|   todo:                                                          |
//|      1. migliorare i setup di entrata e uscita con i patterns    |
//|      2. integrare il money management e calcolo lots max         |
//|                                                                  |
//|   bug:                                                           |
//|      1.                                                          |
//|------------------------------------------------------------------+
#property copyright "andrea"
#property link      "https://www.mql5.com"
#property version   "1.0"


#include <K_Orders.mqh> 
#include <Trade\PositionInfo.mqh>
#include <Trade\Trade.mqh> // trading object
Order order; 
CPositionInfo positionInfo; // trade position object
CTrade cTrade;  

#define EXPERT_MAGIC 1234567890   // MagicNumber of the expert
#resource "\\Indicators\\heiken_ashi.ex5"
#resource "\\Indicators\\Examples\\Custom Moving Average.ex5"

int haHandle=INVALID_HANDLE;
int SMA10Handle=INVALID_HANDLE;
int SMA30Handle=INVALID_HANDLE;

//heiken ashi buffers
double ExtOBuffer[];
double ExtCBuffer[];
double ExtHBuffer[];
double ExtLBuffer[];

//sma buffers
double ExtSMA10Buffer[];
double ExtSMA30Buffer[];

int timeframe = 0;

bool isInBuy = false;
bool isInSell = false;
bool isInTheMarket = false;

int startTime = 0;
int stopTime = 24;

double lots = 5;

int OnInit(){

   if(haHandle=iCustom(_Symbol, _Period, "::Indicators\\heiken_ashi.ex5") == INVALID_HANDLE)
      return(INIT_FAILED);
      
   if(SMA10Handle=iCustom(_Symbol, _Period, "::Indicators\\Examples\\Custom Moving Average.ex5", 10, 0, MODE_SMA) == INVALID_HANDLE)
      return(INIT_FAILED);
      
   if(SMA30Handle=iCustom(_Symbol, _Period, "::Indicators\\Examples\\Custom Moving Average.ex5", 30, 0, MODE_SMA) == INVALID_HANDLE)
      return(INIT_FAILED);
      
   return(INIT_SUCCEEDED);
   
}

void OnDeinit(const int reason){
   
   IndicatorRelease(haHandle);
   IndicatorRelease(SMA10Handle);
   IndicatorRelease(SMA30Handle);
   
}

void OnTick(){

   createBuffers();

   //close the position
   if(isInTheMarket){
      if(isInBuy && isBearCandle()){
         closePosition();
      }else if(isInSell && isBullCandle()){
         closePosition();
      }
   }
      
   //open position
   if(!isInTheMarket){
      if(isMarketTime()){
         if(isBullCandle() && isSMA10OverSMA30()){
            buy();
         }else if (isBearCandle() && isSMA10UnderSMA30()){
            sell();
         }
      }
   }
}

void createBuffers(){

   //heiken ashi indicator
   if(CopyBuffer(haHandle, 0, 0, 3, ExtOBuffer)<=0)
      printf("Error:" + GetLastError() + " error in CopyBuffer ExtOBuffer");
   
   if(CopyBuffer(haHandle, 1, 0, 3, ExtHBuffer)<=0)
      printf("Error:" + GetLastError() + " error in CopyBuffer ExtHBuffer");
      
   if(CopyBuffer(haHandle, 2, 0, 3, ExtLBuffer)<=0)
      printf("Error:" + GetLastError() + " error in CopyBuffer ExtLBuffer");
      
   if(CopyBuffer(haHandle, 3, 0, 3, ExtCBuffer)<=0)
      printf("Error:" + GetLastError() + " error in CopyBuffer ExtCBuffer");
    
   //sma indicators
   if(CopyBuffer(SMA10Handle, 0, 0, 1, ExtSMA10Buffer)<=0)
      printf("Error:" + GetLastError() + " error in CopyBuffer ExtSMA10Buffer");
      
   if(CopyBuffer(SMA30Handle, 0, 0, 1, ExtSMA30Buffer)<=0)
      printf("Error:" + GetLastError() + " error in CopyBuffer ExtSMA30Buffer");
      
}

bool isBullCandle(){
      
   if(checkCandle() == "bull")
      return true;
   else
      return false;
   
}

bool isBearCandle(){
   
   if(checkCandle() == "bear" )
      return true;
   else
      return false;
   
}

string checkCandle(){
   
   string candle = "";
 
   // last finished candle
   double haOpenLastCandle = ExtOBuffer[1];
   double haCloseLastCandle = ExtCBuffer[1];
   double haHighLastCandle = ExtHBuffer[1];
   double haLowLastCandle = ExtLBuffer[1];
      
   if ( (haOpenLastCandle < haCloseLastCandle) && (haOpenLastCandle == haLowLastCandle) ){
      candle = "bull";
   }else if ( (haOpenLastCandle > haCloseLastCandle)  && (haOpenLastCandle == haHighLastCandle) ){
      candle = "bear";
   }
     
   return (candle);

}

bool isSMA10OverSMA30(){

   if(ExtSMA10Buffer[0] > ExtSMA30Buffer[0]){
      //printf(ExtSMA10Buffer[0] +" > " + ExtSMA30Buffer[0] );
      return true;
   }else{
      //printf(ExtSMA10Buffer[0] +" < " + ExtSMA30Buffer[0] );
      return false;
   }
   
}

bool isSMA10UnderSMA30(){

   if(ExtSMA10Buffer[0] < ExtSMA30Buffer[0])
      return true;
   else
      return false;

}

void buy(){
   order.SetLots(lots);
   order.SetMagic(123);
   //order.SetSL_inPoints(20);
   //order.SetTP_inPoints(10);
   if(order.Buy()){
      isInTheMarket = true;
      isInBuy = true;
   }

}

void sell(){
   order.SetLots(lots);
   order.SetMagic(123);
   //order.SetSL_inPoints(20);
   //order.SetTP_inPoints(10);
   if(order.Sell()){
      isInTheMarket = true;
      isInSell = true;
   }
   
}

void closePosition(){
   printf("close position function");
 
   printf("try to close order");
   for(int i=PositionsTotal()-1;i>=0;i--){ // returns the number of current position
      if(positionInfo.SelectByIndex(i))     // selects the position by index for further access to its properties
         if(positionInfo.Symbol()==Symbol())
            if(cTrade.PositionClose(positionInfo.Ticket())){ // close a position by the specified symbol
               isInTheMarket = false;
               isInBuy = false;
               isInSell = false;
               printf("position closed");
            }
   }
    
}

bool isMarketTime(){
   string now = TimeToString(TimeLocal());
   int hour = StringSubstr(now, 11, 2);
   
   if((hour > startTime) && (hour < stopTime))
      return true;
   else
      return false;
   
}
